@@ -14,6 +14,7 @@
 
 #include <linux/btrfs.h>
 #include <sys/ioctl.h>
+#include <sys/utsname.h>
 #include "act_dedupefiles.h"
 
 /* Message to append to BTRFS warnings based on write permissions */
@@ -23,22 +24,21 @@ static const char *readonly_msg[] = {
 };
 
 static char *dedupeerrstr(int err) {
-  static char buf[256];
-
-  buf[sizeof(buf)-1] = '\0';
+  tempname[sizeof(tempname)-1] = '\0';
   if (err == BTRFS_SAME_DATA_DIFFERS) {
-    snprintf(buf, sizeof(buf), "BTRFS_SAME_DATA_DIFFERS (data modified in the meantime?)");
-    return buf;
+    snprintf(tempname, sizeof(tempname), "BTRFS_SAME_DATA_DIFFERS (data modified in the meantime?)");
+    return tempname;
   } else if (err < 0) {
     return strerror(-err);
   } else {
-    snprintf(buf, sizeof(buf), "Unknown error %d", err);
-    return buf;
+    snprintf(tempname, sizeof(tempname), "Unknown error %d", err);
+    return tempname;
   }
 }
 
 extern void dedupefiles(file_t * restrict files)
 {
+  struct utsname utsname;
   struct btrfs_ioctl_same_args *same;
   char **dupe_filenames; /* maps to same->info indices */
 
@@ -51,6 +51,17 @@ extern void dedupefiles(file_t * restrict files)
 
   LOUD(fprintf(stderr, "\nRunning dedupefiles()\n");)
 
+  /* Refuse to dedupe on 2.x kernels; they could damage user data */
+  if (uname(&utsname)) {
+    fprintf(stderr, "Failed to get kernel version! Aborting.\n");
+    exit(EXIT_FAILURE);
+  }
+  LOUD(fprintf(stderr, "dedupefiles: uname got release '%s'\n", utsname.release));
+  if (*(utsname.release) == '2' && *(utsname.release + 1) == '.') {
+    fprintf(stderr, "Refusing to dedupe on a 2.x kernel; data loss could occur. Aborting.\n");
+    exit(EXIT_FAILURE);
+  }
+
   /* Find the largest dupe set, alloc space to hold structs for it */
   get_max_dupes(files, &max_dupes, &max_files);
   /* Kernel dupe count is a uint16_t so exit if the type's limit is exceeded */
@@ -116,7 +127,7 @@ extern void dedupefiles(file_t * restrict files)
       n_dupes = cur_info;
 
       same->logical_offset = 0;
-      same->length = (unsigned long)files->size;
+      same->length = (uint64_t)files->size;
       same->dest_count = (uint16_t)n_dupes;  /* kernel type is __u16 */
 
       fd = open(files->d_name, O_RDONLY);
