@@ -41,14 +41,7 @@
 #include <sys/time.h>
 #include "jdupes.h"
 #include "string_malloc.h"
-#if defined USE_HASH_JODYHASH
- #define JODY_HASH_NOCOMPAT
- #include "jody_hash.h"
-#elif defined USE_HASH_XXHASH64
- #include "xxhash.h"
-#else
- #error No USE_HASH is defined
-#endif
+#include "xxhash.h"
 #include "jody_sort.h"
 #include "jody_win_unicode.h"
 #include "jody_cacheinfo.h"
@@ -61,7 +54,6 @@
 #include "act_printmatches.h"
 #include "act_summarize.h"
 
-
 /* Detect Windows and modify as needed */
 #if defined _WIN32 || defined __CYGWIN__
  const char dir_sep = '\\';
@@ -82,13 +74,9 @@
 
 /* Windows + Unicode compilation */
 #ifdef UNICODE
-wchar_t wname[PATH_MAX];
-wchar_t wname2[PATH_MAX];
-wchar_t wstr[PATH_MAX];
+wpath_t wname,wname2,wstr;
 int out_mode = _O_TEXT;
 int err_mode = _O_TEXT;
- #define M2W(a,b) MultiByteToWideChar(CP_UTF8, 0, a, -1, (LPWSTR)b, PATH_MAX)
- #define W2M(a,b) WideCharToMultiByte(CP_UTF8, 0, a, -1, (LPSTR)b, PATH_MAX, NULL, NULL)
 #endif /* UNICODE */
 
 #ifndef NO_SYMLINKS
@@ -133,13 +121,6 @@ static size_t auto_chunk_size = CHUNK_SIZE;
 #error "PATHBUF_SIZE can't be less than PATH_MAX"
 #endif
 
-#ifndef INITIAL_DEPTH_THRESHOLD
-#define INITIAL_DEPTH_THRESHOLD 8
-#endif
-
-/* For interactive deletion input */
-#define INPUT_SIZE 512
-
 /* Size suffixes - this gets exported */
 const struct size_suffix size_suffix[] = {
   /* Byte (someone may actually try to use this) */
@@ -195,20 +176,6 @@ static const char *extensions[] = {
     #ifdef SMA_PAGE_SIZE
     "smapage",
     #endif
-    #ifdef USE_JODY_HASH
-    #if JODY_HASH_WIDTH == 64
-    "jodyhash64",
-    #endif
-    #if JODY_HASH_WIDTH == 32
-    "jodyhash32",
-    #endif
-    #if JODY_HASH_WIDTH == 16
-    "jodyhash16",
-    #endif
-    #endif /* USE_JODY_HASH */
-    #ifdef USE_HASH_XXHASH64
-    "xxhash64",
-    #endif
     #ifdef NO_PERMS
     "noperm",
     #endif
@@ -287,6 +254,8 @@ static int interrupt = 0;
 /* Progress indicator time */
 struct timeval time1, time2;
 
+/* For path name mangling */
+char tempname[PATHBUF_SIZE * 2];
 
 /***** End definitions, begin code *****/
 
@@ -304,6 +273,16 @@ void sighandler(const int signum)
   return;
 }
 
+#ifndef ON_WINDOWS
+void sigusr1(const int signum)
+{
+  (void)signum;
+  if (!ISFLAG(flags, F_SOFTABORT)) SETFLAG(flags, F_SOFTABORT);
+  else CLEARFLAG(flags, F_SOFTABORT);
+  return;
+}
+#endif
+
 
 /* Out of memory */
 extern void oom(const char * const restrict msg)
@@ -324,7 +303,7 @@ extern void nullptr(const char * restrict func)
   exit(EXIT_FAILURE);
 }
 
-/* Compare two jody_hashes like memcmp() */
+/* Compare two hashes like memcmp() */
 #define HASH_COMPARE(a,b) ((a > b) ? 1:((a == b) ? 0:-1))
 
 
@@ -409,6 +388,9 @@ static void update_progress(const char * const restrict msg, const int file_perc
  * Returns 1 if changed, 0 if not changed, negative if error */
 extern int file_has_changed(file_t * const restrict file)
 {
+  /* If -t/--no-tocttou specified then completely bypass this code */
+  if (ISFLAG(flags, F_NO_TOCTTOU)) return 0;
+
   if (file == NULL || file->d_name == NULL) nullptr("file_has_changed()");
   LOUD(fprintf(stderr, "file_has_changed('%s')\n", file->d_name);)
 
@@ -665,7 +647,6 @@ extern int check_conditions(const file_t * const restrict file1, const file_t *
 /* Check for exclusion conditions for a single file (1 = fail) */
 static int check_singlefile(file_t * const restrict newfile)
 {
-  static char tempname[PATHBUF_SIZE * 2];
   char * restrict tp = tempname;
   int excluded;
 
@@ -809,7 +790,6 @@ static void grokdir(const char * const restrict dir,
   file_t * restrict newfile;
   struct dirent *dirinfo;
   static int grokdir_level = 0;
-  static char tempname[PATHBUF_SIZE * 2];
   size_t dirlen;
   struct travdone *traverse;
   int i, single = 0;
@@ -882,16 +862,16 @@ static void grokdir(const char * const restrict dir,
 
 #ifdef UNICODE
   /* Windows requires \* at the end of directory names */
-  strncpy(tempname, dir, PATHBUF_SIZE * 2);
+  strncpy(tempname, dir, PATHBUF_SIZE * 2 - 1);
   dirlen = strlen(tempname) - 1;
   p = tempname + dirlen;
   if (*p == '/' || *p == '\\') *p = '\0';
-  strncat(tempname, "\\*", PATHBUF_SIZE * 2);
+  strncat(tempname, "\\*", PATHBUF_SIZE * 2 - 1);
 
   if (!M2W(tempname, wname)) goto error_cd;
 
   LOUD(fprintf(stderr, "FindFirstFile: %s\n", dir));
-  hFind = FindFirstFile((LPCWSTR)wname, &ffd);
+  hFind = FindFirstFileW(wname, &ffd);
   if (hFind == INVALID_HANDLE_VALUE) { LOUD(fprintf(stderr, "\nfile handle bad\n")); goto error_cd; }
   LOUD(fprintf(stderr, "Loop start\n"));
   do {
@@ -942,6 +922,8 @@ static void grokdir(const char * const restrict dir,
     tp = tempname;
     memcpy(newfile->d_name, tp, dirlen + d_name_len);
 
+    /*** WARNING: tempname global gets reused by check_singlefile here! ***/
+
     /* Single-file [l]stat() and exclusion condition check */
     if (check_singlefile(newfile) != 0) {
       LOUD(fprintf(stderr, "grokdir: check_singlefile rejected file\n"));
@@ -1008,7 +990,7 @@ add_single_file:
   }
 
 #ifdef UNICODE
-  while (FindNextFile(hFind, &ffd) != 0);
+  while (FindNextFileW(hFind, &ffd) != 0);
   FindClose(hFind);
 #else
   closedir(cd);
@@ -1044,9 +1026,7 @@ static jdupes_hash_t *get_filehash(const file_t * const restrict checkfile,
   static jdupes_hash_t *chunk = NULL;
   FILE *file;
   int check = 0;
-#ifdef USE_HASH_XXHASH64
   XXH64_state_t *xxhstate;
-#endif
 
   if (checkfile == NULL || checkfile->d_name == NULL) nullptr("get_filehash()");
   LOUD(fprintf(stderr, "get_filehash('%s', %" PRIdMAX ")\n", checkfile->d_name, (intmax_t)max_read);)
@@ -1106,11 +1086,9 @@ static jdupes_hash_t *get_filehash(const file_t * const restrict checkfile,
     fsize -= PARTIAL_HASH_SIZE;
   }
 
-#ifdef USE_HASH_XXHASH64
   xxhstate = XXH64_createState();
   if (xxhstate == NULL) nullptr("xxhstate");
   XXH64_reset(xxhstate, 0);
-#endif
 
   /* Read the file in CHUNK_SIZE chunks until we've read it all. */
   while (fsize > 0) {
@@ -1124,11 +1102,7 @@ static jdupes_hash_t *get_filehash(const file_t * const restrict checkfile,
       return NULL;
     }
 
-#if defined USE_HASH_JODYHASH
-    *hash = jody_block_hash(chunk, *hash, bytes_to_read);
-#elif defined USE_HASH_XXHASH64
     XXH64_update(xxhstate, chunk, bytes_to_read);
-#endif
 
     if ((off_t)bytes_to_read > fsize) break;
     else fsize -= (off_t)bytes_to_read;
@@ -1144,10 +1118,9 @@ static jdupes_hash_t *get_filehash(const file_t * const restrict checkfile,
 
   fclose(file);
 
-#ifdef USE_HASH_XXHASH64
   *hash = XXH64_digest(xxhstate);
   XXH64_freeState(xxhstate);
-#endif
+
   LOUD(fprintf(stderr, "get_filehash: returning hash: 0x%016jx\n", (uintmax_t)*hash));
   return hash;
 }
@@ -1256,8 +1229,13 @@ static file_t **checkmatch(filetree_t * restrict tree, file_t * const restrict f
     LOUD(if (cmpresult) fprintf(stderr, "checkmatch: partial hashes do not match\n"));
     DBG(partial_hash++;)
 
-    if (file->size <= PARTIAL_HASH_SIZE) {
-      LOUD(fprintf(stderr, "checkmatch: small file: copying partial hash to full hash\n"));
+    /* Print partial hash matching pairs if requested */
+    if (cmpresult == 0 && ISFLAG(p_flags, P_PARTIAL))
+      printf("Partial hashes match:\n   %s\n   %s\n\n", file->d_name, tree->file->d_name);
+
+    if (file->size <= PARTIAL_HASH_SIZE || ISFLAG(flags, F_PARTIALONLY)) {
+      if (ISFLAG(flags, F_PARTIALONLY)) LOUD(fprintf(stderr, "checkmatch: partial only mode: treating partial hash as full hash\n"));
+      else { LOUD(fprintf(stderr, "checkmatch: small file: copying partial hash to full hash\n")); }
       /* filehash_partial = filehash if file is small enough */
       if (!ISFLAG(file->flags, F_HASH_FULL)) {
         file->filehash = file->filehash_partial;
@@ -1287,9 +1265,6 @@ static file_t **checkmatch(filetree_t * restrict tree, file_t * const restrict f
         SETFLAG(file->flags, F_HASH_FULL);
       }
 
-      /* Print partial hash matching pairs if requested */
-      if (ISFLAG(p_flags, P_PARTIAL)) printf("Partial hashes match:\n   %s\n   %s\n\n", file->d_name, tree->file->d_name);
-
       /* Full file hash comparison */
       cmpresult = HASH_COMPARE(file->filehash, tree->file->filehash);
       LOUD(if (!cmpresult) fprintf(stderr, "checkmatch: full hashes match\n"));
@@ -1337,7 +1312,7 @@ static file_t **checkmatch(filetree_t * restrict tree, file_t * const restrict f
 
 /* Do a byte-by-byte comparison in case two different files produce the
    same signature. Unlikely, but better safe than sorry. */
-static inline int confirmmatch(FILE * const restrict file1, FILE * const restrict file2, off_t size)
+static inline int confirmmatch(FILE * const restrict file1, FILE * const restrict file2, const off_t size)
 {
   static char *c1 = NULL, *c2 = NULL;
   size_t r1, r2;
@@ -1496,11 +1471,14 @@ static void registerpair(file_t **matchlist, file_t *newmatch,
 
 static inline void help_text(void)
 {
-  printf("Usage: jdupes [options] DIRECTORY...\n\n");
+  printf("Usage: jdupes [options] FILES and/or DIRECTORIES...\n\n");
 
+  printf("Duplicate file sets will be printed by default unless a different action\n");
+  printf("option is specified (delete, summarize, link, dedupe, etc.)\n");
 #ifdef LOUD
   printf(" -@ --loud        \toutput annoying low-level debug info while running\n");
 #endif
+  printf(" -0 --printnull   \toutput nulls instead of CR/LF (like 'find -print0')\n");
   printf(" -1 --one-file-system \tdo not match files on different filesystems/devices\n");
   printf(" -A --nohidden    \texclude hidden files from consideration\n");
 #ifdef ENABLE_BTRFS
@@ -1536,7 +1514,7 @@ static inline void help_text(void)
  #endif /* ON_WINDOWS */
 #endif /* NO_HARDLINKS */
   printf(" -m --summarize   \tsummarize dupe information\n");
-  //printf(" -n --noempty     \texclude zero-length files from consideration\n");
+  printf(" -M --printwithsummary\twill print matches and --summarize at the end\n");
   printf(" -N --noprompt    \ttogether with --delete, preserve the first file in\n");
   printf("                  \teach set of duplicates and delete the rest without\n");
   printf("                  \tprompting the user\n");
@@ -1550,6 +1528,7 @@ static inline void help_text(void)
   printf("                  \tpermission bits as duplicates\n");
 #endif
   printf(" -P --print=type  \tprint extra info (partial, early, fullhash)\n");
+  printf(" -q --quiet       \thide progress indicator\n");
   printf(" -Q --quick       \tskip byte-for-byte confirmation for quick matching\n");
   printf("                  \tWARNING: -Q can result in data loss! Be very careful!\n");
   printf(" -r --recurse     \tfor every directory, process its subdirectories too\n");
@@ -1560,11 +1539,10 @@ static inline void help_text(void)
   printf(" -s --symlinks    \tfollow symlinks\n");
 #endif
   printf(" -S --size        \tshow size of duplicate files\n");
-  printf(" -q --quiet       \thide progress indicator\n");
-/* This is undocumented in the quick help because it is a dangerous option. If you
- * really want it, uncomment it here, and may your data rest in peace. */
-/*  printf(" -Q --quick       \tskip byte-by-byte duplicate verification. WARNING:\n");
-  printf("                  \tthis may delete non-duplicates! Read the manual first!\n"); */
+  printf(" -t --no-tocttou  \tdisable security check for file changes (aka TOCTTOU)\n");
+  printf(" -T --partial-only \tmatch based on partial hashes only. WARNING:\n");
+  printf("                  \tEXTREMELY DANGEROUS paired with destructive actions!\n");
+  printf("                  \t-T must be specified twice to work. Read the manual!\n");
   printf(" -v --version     \tdisplay jdupes version and license information\n");
   printf(" -x --xsize=SIZE  \texclude files of size < SIZE bytes from consideration\n");
   printf("    --xsize=+SIZE \t'+' specified before SIZE, exclude size > SIZE\n");
@@ -1573,6 +1551,9 @@ static inline void help_text(void)
   printf("                  \tExclusions are cumulative: -X dir:abc -X dir:efg\n");
   printf(" -z --zeromatch   \tconsider zero-length files to be duplicates\n");
   printf(" -Z --softabort   \tIf the user aborts (i.e. CTRL-C) act on matches so far\n");
+#ifndef ON_WINDOWS
+  printf("                  \tYou can send SIGUSR1 to the program to toggle this\n");
+#endif
   printf("\nFor sizes, K/M/G/T/P/E[B|iB] suffixes can be used (case-insensitive)\n");
 #ifdef OMIT_GETOPT_LONG
   printf("Note: Long options are not supported in this build.\n\n");
@@ -1593,6 +1574,7 @@ int main(int argc, char **argv)
   static int firstrecurse;
   static int opt;
   static int pm = 1;
+  static int partialonly_spec = 0;
   static ordertype_t ordertype = ORDER_NAME;
   static long manual_chunk_size = 0;
 #ifndef ON_WINDOWS
@@ -1603,6 +1585,7 @@ int main(int argc, char **argv)
   static const struct option long_options[] =
   {
     { "loud", 0, 0, '@' },
+    { "printnull", 0, 0, '0' },
     { "one-file-system", 0, 0, '1' },
     { "nohidden", 0, 0, 'A' },
     { "dedupe", 0, 0, 'B' },
@@ -1617,7 +1600,7 @@ int main(int argc, char **argv)
     { "linksoft", 0, 0, 'l' },
     { "linkhard", 0, 0, 'L' },
     { "summarize", 0, 0, 'm'},
-    { "summary", 0, 0, 'm' },
+    { "printwithsummary", 0, 0, 'M'},
     { "noempty", 0, 0, 'n' },
     { "noprompt", 0, 0, 'N' },
     { "order", 1, 0, 'o' },
@@ -1632,6 +1615,8 @@ int main(int argc, char **argv)
     { "recursive:", 0, 0, 'R' },
     { "symlinks", 0, 0, 's' },
     { "size", 0, 0, 'S' },
+    { "no-tocttou", 0, 0, 't' },
+    { "partial-only", 0, 0, 'T' },
     { "version", 0, 0, 'v' },
     { "xsize", 1, 0, 'x' },
     { "exclude", 1, 0, 'X' },
@@ -1677,17 +1662,27 @@ int main(int argc, char **argv)
     auto_chunk_size = (auto_chunk_size + 0x00000fffUL) & 0x000ff000;
 #endif /* ON_WINDOWS */
 
+  /* Is stderr a terminal? If not, we won't write progress to it */
+#ifdef ON_WINDOWS
+  if (!_isatty(_fileno(stderr))) SETFLAG(flags, F_HIDEPROGRESS);
+#else
+  if (!isatty(fileno(stderr))) SETFLAG(flags, F_HIDEPROGRESS);
+#endif
+
   program_name = argv[0];
 
   oldargv = cloneargs(argc, argv);
 
   while ((opt = GETOPT(argc, argv,
-  "@1ABC:dDfhHiIlLmnNOpP:qQrRsSvzZo:x:X:"
+  "@01ABC:dDfhHiIlLmMnNOpP:qQrRsStTvVzZo:x:X:"
 #ifndef OMIT_GETOPT_LONG
           , long_options, NULL
 #endif
          )) != EOF) {
     switch (opt) {
+    case '0':
+      SETFLAG(flags, F_PRINTNULL);
+      break;
     case '1':
       SETFLAG(flags, F_ONEFS);
       break;
@@ -1745,6 +1740,10 @@ int main(int argc, char **argv)
     case 'm':
       SETFLAG(flags, F_SUMMARIZEMATCHES);
       break;
+    case 'M':
+      SETFLAG(flags, F_SUMMARIZEMATCHES);
+      SETFLAG(flags, F_PRINTMATCHES);
+      break;
     case 'n':
       //fprintf(stderr, "note: -n/--noempty is the default behavior now and is deprecated.\n");
       break;
@@ -1775,6 +1774,17 @@ int main(int argc, char **argv)
     case 'R':
       SETFLAG(flags, F_RECURSEAFTER);
       break;
+    case 't':
+      SETFLAG(flags, F_NO_TOCTTOU);
+      break;
+    case 'T':
+      if (partialonly_spec == 0)
+        partialonly_spec = 1;
+      else {
+        partialonly_spec = 2;
+        SETFLAG(flags, F_PARTIALONLY);
+      }
+      break;
 #ifndef NO_SYMLINKS
     case 'l':
       SETFLAG(flags, F_MAKESYMLINKS);
@@ -1796,7 +1806,7 @@ int main(int argc, char **argv)
       fprintf(stderr, "-x/--xsize is deprecated; use -X size[+-=]:size[suffix] instead\n");
       xs = string_malloc(8 + strlen(optarg));
       if (xs == NULL) oom("xsize temp string");
-      strcat(xs, "size");
+      strcpy(xs, "size");
       if (*optarg == '+') {
         strcat(xs, "+:");
         optarg++;
@@ -1816,6 +1826,7 @@ int main(int argc, char **argv)
 #endif
       break;
     case 'v':
+    case 'V':
       printf("jdupes %s (%s) ", VER, VERDATE);
 
       /* Indicate bitness information */
@@ -1828,6 +1839,11 @@ int main(int argc, char **argv)
       } else printf("%u-bit i%u\n", (unsigned int)(sizeof(uintptr_t) * 8),
           (unsigned int)(sizeof(long) * 8));
 
+#ifdef BUILD_DATE
+#include "build_date.h"
+      printf("Built on %s\n", BUILT_ON_DATE);
+#endif
+
       printf("Compile-time extensions:");
       if (*extensions != NULL) {
         int c = 0;
@@ -1886,7 +1902,19 @@ int main(int argc, char **argv)
   }
 
   if (optind >= argc) {
-    fprintf(stderr, "no directories specified (use -h option for help)\n");
+    fprintf(stderr, "no files or directories specified (use -h option for help)\n");
+    string_malloc_destroy();
+    exit(EXIT_FAILURE);
+  }
+
+  if (partialonly_spec == 1) {
+    fprintf(stderr, "--partial-only specified only once (it's VERY DANGEROUS, read the manual!)\n");
+    string_malloc_destroy();
+    exit(EXIT_FAILURE);
+  }
+
+  if (ISFLAG(flags, F_PARTIALONLY) && ISFLAG(flags, F_QUICKCOMPARE)) {
+    fprintf(stderr, "--partial-only overrides --quick and is even more dangerous (read the manual!)\n");
     string_malloc_destroy();
     exit(EXIT_FAILURE);
   }
@@ -1916,7 +1944,7 @@ int main(int argc, char **argv)
       !!ISFLAG(flags, F_DEDUPEFILES);
 
   if (pm > 1) {
-      fprintf(stderr, "Only one of --summarize, --delete, --linkhard, --linksoft, or --dedupe\nmay be used\n");
+      fprintf(stderr, "Only one of --summarize, --printwithsummary, --delete,\n--linkhard, --linksoft, or --dedupe may be used\n");
       string_malloc_destroy();
       exit(EXIT_FAILURE);
   }
@@ -1969,6 +1997,10 @@ int main(int argc, char **argv)
 
   /* Catch CTRL-C */
   signal(SIGINT, sighandler);
+#ifndef ON_WINDOWS
+  /* Catch SIGUSR1 and use it to enable -Z */
+  signal(SIGUSR1, sigusr1);
+#endif
 
   while (curfile) {
     static file_t **match = NULL;
@@ -1989,15 +2021,15 @@ int main(int argc, char **argv)
 
     /* Byte-for-byte check that a matched pair are actually matched */
     if (match != NULL) {
-      /* Quick comparison mode will never run confirmmatch()
+      /* Quick or partial-only compare will never run confirmmatch()
        * Also skip match confirmation for hard-linked files
        * (This set of comparisons is ugly, but quite efficient) */
-      if (ISFLAG(flags, F_QUICKCOMPARE) ||
+      if (ISFLAG(flags, F_QUICKCOMPARE) || ISFLAG(flags, F_PARTIALONLY) ||
            (ISFLAG(flags, F_CONSIDERHARDLINKS) &&
            (curfile->inode == (*match)->inode) &&
            (curfile->device == (*match)->device))
          ) {
-        LOUD(fprintf(stderr, "MAIN: notice: quick compare match (-Q)\n"));
+        LOUD(fprintf(stderr, "MAIN: notice: quick or partial-only match (-Q/-T)\n"));
         registerpair(match, curfile,
             (ordertype == ORDER_TIME) ? sort_pairs_by_mtime : sort_pairs_by_filename);
         dupecount++;
@@ -2054,7 +2086,6 @@ skip_file_scan:
     if (ISFLAG(flags, F_NOPROMPT)) deletefiles(files, 0, 0);
     else deletefiles(files, 1, stdin);
   }
-  if (ISFLAG(flags, F_SUMMARIZEMATCHES)) summarizematches(files);
 #ifndef NO_SYMLINKS
   if (ISFLAG(flags, F_MAKESYMLINKS)) linkfiles(files, 0);
 #endif
@@ -2065,6 +2096,10 @@ skip_file_scan:
   if (ISFLAG(flags, F_DEDUPEFILES)) dedupefiles(files);
 #endif /* ENABLE_BTRFS */
   if (ISFLAG(flags, F_PRINTMATCHES)) printmatches(files);
+  if (ISFLAG(flags, F_SUMMARIZEMATCHES)) {
+    if (ISFLAG(flags, F_PRINTMATCHES)) printf("\n\n");
+    summarizematches(files);
+  }
 
   string_malloc_destroy();
 
