@@ -8,18 +8,6 @@
 extern "C" {
 #endif
 
-/* Select hash algorithm */
-//#define USE_HASH_JODYHASH /* jodyhash */
-//#define USE_HASH_XXHASH64 /* xxHash64 */
-
-/* Failsafes */
-#if !defined USE_HASH_JODYHASH && !defined USE_HASH_XXHASH64
-#define USE_HASH_JODYHASH
-#endif
-#if defined USE_HASH_JODYHASH && defined USE_HASH_XXHASH64
-#error Multiple USE_HASH options
-#endif
-
 /* Detect Windows and modify as needed */
 #if defined _WIN32 || defined __CYGWIN__
  #ifndef ON_WINDOWS
@@ -46,13 +34,7 @@ extern "C" {
 #include "jody_sort.h"
 #include "version.h"
 
-/* Configure hash function based on choice above */
-#if defined USE_HASH_JODYHASH
- #define JODY_HASH_NOCOMPAT
- #include "jody_hash.h"
-#elif defined USE_HASH_XXHASH64
- #include "xxhash.h"
-#endif
+#include "xxhash.h"
 
 /* Optional btrfs support */
 #ifdef ENABLE_BTRFS
@@ -61,11 +43,7 @@ extern "C" {
 #endif
 
 /* Set hash type (change this if swapping in a different hash function) */
-#if defined USE_HASH_JODYHASH
- typedef jodyhash_t jdupes_hash_t;
-#elif defined USE_HASH_XXHASH64
  typedef XXH64_hash_t jdupes_hash_t;
-#endif
 
 /* Some types are different on Windows */
 #ifdef ON_WINDOWS
@@ -92,13 +70,14 @@ extern "C" {
 
 /* Windows + Unicode compilation */
 #ifdef UNICODE
-extern wchar_t wname[PATH_MAX];
-extern wchar_t wname2[PATH_MAX];
-extern wchar_t wstr[PATH_MAX];
-extern int out_mode;
-extern int err_mode;
- #define M2W(a,b) MultiByteToWideChar(CP_UTF8, 0, a, -1, (LPWSTR)b, PATH_MAX)
- #define W2M(a,b) WideCharToMultiByte(CP_UTF8, 0, a, -1, (LPSTR)b, PATH_MAX, NULL, NULL)
+ #define WPATH_MAX 8192
+ #define PATHBUF_SIZE WPATH_MAX
+  typedef wchar_t wpath_t[WPATH_MAX];
+  extern wpath_t wname,wname2,wstr;
+  extern int out_mode;
+  extern int err_mode;
+ #define M2W(a,b) MultiByteToWideChar(CP_UTF8, 0, a, -1, (LPWSTR)b, WPATH_MAX)
+ #define W2M(a,b) WideCharToMultiByte(CP_UTF8, 0, a, -1, (LPSTR)b, WPATH_MAX, NULL, NULL)
 #endif /* UNICODE */
 
 #ifndef NO_SYMLINKS
@@ -137,9 +116,6 @@ extern int err_mode;
 #endif
 
 
-/* How many operations to wait before updating progress counters */
-#define DELAY_COUNT 256
-
 /* Behavior modification flags */
 extern uint_fast32_t flags;
 #define F_RECURSE		0x00000001U
@@ -166,6 +142,9 @@ extern uint_fast32_t flags;
 #define F_MAKESYMLINKS		0x00200000U
 #define F_PRINTMATCHES		0x00400000U
 #define F_ONEFS			0x00800000U
+#define F_PRINTNULL		0x01000000U
+#define F_PARTIALONLY		0x02000000U
+#define F_NO_TOCTTOU		0x04000000U
 
 #define F_LOUD			0x40000000U
 #define F_DEBUG			0x80000000U
@@ -201,9 +180,6 @@ typedef enum {
 #define PATHBUF_SIZE 4096
 #endif
 
-/* For interactive deletion input */
-#define INPUT_SIZE 512
-
 /* Per-file information */
 typedef struct _file {
   struct _file *duplicates;
@@ -288,7 +264,7 @@ struct size_suffix {
 };
 
 extern const struct size_suffix size_suffix[];
-
+extern char tempname[PATHBUF_SIZE * 2];
 
 extern void oom(const char * const restrict msg);
 extern void nullptr(const char * restrict func);
